#include "mcpl.h"
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>

mcpl_file_t f;
int count;

int signbit(double a){
  return(a<0)?1:0;
}

void read_init() 
{  printf("Message from C read initialization routine\n");
   printf(">>Opening file W4neutrons1M.mcpl\n");
   f = mcpl_open_file("../W4neutrons1M.mcpl");
   printf(">>File contains %llu particles\n",(unsigned long long)mcpl_hdr_nparticles(f));
   int i;
   for (i = 0; i < mcpl_hdr_ncomments(f); ++i)
   printf(">>file had comment: '%s'\n",mcpl_hdr_comment(f,i));
   count=0;
   double totalweight=0.0;
   const mcpl_particle_t* p;
   while ( p = mcpl_read(f))  totalweight = p->weight;
   mcpl_seek(f,0);
   printf(">>End of file reached, total weight of primaries = %.10g\n",totalweight);   
}

//void read_do (double p, double t, double w){
int read_do (){
  const mcpl_particle_t* p;
  if ( p = mcpl_read(f)){
    //print some info (see the mcpl_particle_t struct in mcpl.h for more fields):
    printf("  found particle with pdgcode %i and time-stamp %g ms with weight %g\n",
           p->pdgcode, p->time, p->weight);
    return 0;
  } else return 1;

}

int getno() {
  return (int) mcpl_hdr_nparticles(f);
}

//void read_do (double p, double t, double w){
void read_circled (double* ekin, double* weight, 
		   double* x, double* y, double* z, 
		   double* tx,  double* ty, double* tz,
		   double* sx,  double*sy,  double*sz, 
		   double* time, int* pdgcode){
  const mcpl_particle_t* p;
  if (!( p = mcpl_read(f))){
   printf("End of file reached, restarting\n"); 
   mcpl_seek(f,0);
   p = mcpl_read(f);
   //   printf("  found particle with pdgcode %i and time-stamp %g ms with weight %g\n",
   //         p->pdgcode, p->time, p->weight);
  }
  *ekin = p->ekin;
  *weight = p->weight;
  *x = p->position[0];
  *y = p->position[1];
  *z = p->position[2];
  *tx = p->direction[0];
  *ty = p->direction[1];
  *tz = p->direction[2];
  *sx = p->polarisation[0];
  *sy = p->polarisation[1];
  *sz = p->polarisation[2];
  *time = p->time;
  *pdgcode = (int) p->pdgcode;  
  //    double ekin;            /* kinetic energy [MeV]             */
  //    double polarisation[3]; /* polarisation vector              */
  //    double position[3];     /* position [cm]                    */
  //    double direction[3];    /* momentum direction (unit vector) */
  //    double time;            /* time-stamp [millisecond]         */
  //    double weight;          /* weight or intensity              */
  //    int32_t pdgcode; /* MC particle number from the Particle Data Group (2112=neutron, 22=gamma, ...)        */

}


void read_stop()
{
  printf("Closing file...\n");
  mcpl_close_file(f);
}
